<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Collection;

use App\Traits\ScopeActive;

class Domain extends Model
{
	use HasFactory,ScopeActive;
	private const CACHE_TIME = 3600;

	/* SCOPES */

	/**
	 * Only query active records
	 */
	public function scopePublic($query)
	{
		return $query->where('public',TRUE);
	}

	/* RELATIONS */

	public function echoareas()
	{
		return $this->hasMany(Echoarea::class);
	}

	public function fileareas()
	{
		return $this->hasMany(Filearea::class);
	}

	public function zones()
	{
		return $this->hasMany(Zone::class);
	}

	/* CASTS */

	public function getHomePageAttribute($value)
	{
		return $value ? gzuncompress(base64_decode($value)) : 'No available information at the moment.';
	}

	public function setHomePageAttribute($value)
	{
		$this->attributes['homepage'] = base64_encode(gzcompress($value,9));
	}

	/* METHODS */

	public function daily_area_stats(): Collection
	{
		if (! $this->echoareas->count())
			return collect();

		$key = sprintf('%s_%d','daily_area_stats',$this->id);

		return Cache::driver('redis')->remember($key,self::CACHE_TIME,function() {
			$where = ['echoarea_id'=>$this->echoareas->pluck('id')->toArray()];

			$echostats = Echomail::countGroupBy(['datetime',['datetime'=>'%Y-%m-%d']],$where);

			return $echostats
				->sortBy(function($item) { return $item->id->datetime; })
				->map(function($item) { return ['x'=>Carbon::createFromFormat('Y-m-d',$item->id->datetime)->timestamp*1000,'y'=>$item->count]; })
				->values();
		});
	}

	public function daily_echoarea_stats(Echoarea $o): Collection
	{
		if (! $this->echoareas->count())
			return collect();

		$key = sprintf('%s_%d-%d','daily_echoarea_stats',$this->id,$o->id);

		return Cache::driver('redis')->remember($key,self::CACHE_TIME,function() use ($o) {
			$where = ['echoarea_id'=>[$o->id]];

			$echostats = Echomail::countGroupBy(['datetime',['datetime'=>'%Y-%m-%d']],$where);

			return $echostats
				->sortBy(function($item) { return $item->id->datetime; })
				->map(function($item) { return ['x'=>Carbon::createFromFormat('Y-m-d',$item->id->datetime)->timestamp*1000,'y'=>$item->count]; })
				->values();
		});
	}

	public function stats(System $o=NULL): Collection
	{
		if (! $this->echoareas->count())
			return collect();

		$key = sprintf('%s_%d_%d','stats',$this->id,$o?->id);

		return Cache::driver('redis')->remember($key,self::CACHE_TIME,function() use ($o) {
			$where = collect(['echoarea_id'=>$this->echoareas->pluck('id')->toArray()]);

			if ($o)
				$where->put('fftn_id',$o->addresses()->pluck('id'));

			$echostats = Echomail::countGroupBy(['echoarea_id'],$where->toArray());

			return $this->echoareas->map(function($item) use ($echostats) {
				$stats = $echostats->filter(function($x) use ($item) {
					return $x->id->echoarea_id == $item->id;
				});

				$item->count = 0;

				foreach ($stats as $o)
					$item->count += $o->count;

				return $item;
			});
		});
	}
}